function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var react = require('react');
var createActivityDetector = _interopDefault(require('activity-detector'));

var usePageWidth = function usePageWidth() {
  var _useState = react.useState(window.innerWidth),
      pageWidth = _useState[0],
      setPageWidth = _useState[1];

  var updateDimensions = function updateDimensions() {
    setPageWidth(window.innerWidth);
  };

  react.useEffect(function () {
    window.addEventListener('resize', updateDimensions);
    return function () {
      window.removeEventListener('resize', updateDimensions);
    };
  }, [pageWidth]);
  return pageWidth;
};

// A type of promise-like that resolves synchronously and supports only one observer

const _iteratorSymbol = /*#__PURE__*/ typeof Symbol !== "undefined" ? (Symbol.iterator || (Symbol.iterator = Symbol("Symbol.iterator"))) : "@@iterator";

const _asyncIteratorSymbol = /*#__PURE__*/ typeof Symbol !== "undefined" ? (Symbol.asyncIterator || (Symbol.asyncIterator = Symbol("Symbol.asyncIterator"))) : "@@asyncIterator";

// Asynchronously call a function and send errors to recovery continuation
function _catch(body, recover) {
	try {
		var result = body();
	} catch(e) {
		return recover(e);
	}
	if (result && result.then) {
		return result.then(void 0, recover);
	}
	return result;
}

var randomJokeApi = 'https://api.jsonbin.io/b/5eda3e691f9e4e578817d386';

var errorMessage = 'Upps...Something went wrong, please try later.';

var getRandomIndex = function getRandomIndex(array) {
  return Math.floor(Math.random() * array.length);
};

var useHelloRomanJoke = function useHelloRomanJoke() {
  var _useState = react.useState(),
      randomQuestion = _useState[0],
      setRandomQuestion = _useState[1];

  var _useState2 = react.useState(),
      randomAnswer = _useState2[0],
      setRandomAnswer = _useState2[1];

  var _useState3 = react.useState(),
      error = _useState3[0],
      setError = _useState3[1];

  var generateRandomJoke = function generateRandomJoke(jokes) {
    var _jokes$getRandomIndex = jokes[getRandomIndex(jokes)],
        question = _jokes$getRandomIndex.question,
        answer = _jokes$getRandomIndex.answer;
    setRandomQuestion(question);
    setRandomAnswer(answer);
  };

  var fetchJokes = function fetchJokes() {
    try {
      var _temp2 = _catch(function () {
        return Promise.resolve(fetch(randomJokeApi)).then(function (data) {
          return Promise.resolve(data.json()).then(function (_ref) {
            var jokes = _ref.jokes;
            generateRandomJoke(jokes);
          });
        });
      }, function () {
        setError(errorMessage);
      });

      return Promise.resolve(_temp2 && _temp2.then ? _temp2.then(function () {}) : void 0);
    } catch (e) {
      return Promise.reject(e);
    }
  };

  react.useEffect(function () {
    fetchJokes();
  }, []);
  return [randomQuestion, randomAnswer, error];
};

var useUnsplash = function useUnsplash(_ref) {
  var user = _ref.user,
      _ref$random = _ref.random,
      random = _ref$random === void 0 ? true : _ref$random,
      width = _ref.width,
      height = _ref.height;

  var _useState = react.useState(),
      randomImage = _useState[0],
      setRandomImage = _useState[1];

  var _useState2 = react.useState(),
      error = _useState2[0],
      setError = _useState2[1];

  var fetchImage = function fetchImage() {
    try {
      try {
        var apiLink = "https://source.unsplash.com/" + (user ? "user/" + user : random ? 'random' : null) + "/" + (width || height ? width + "x" + height : null);
        setRandomImage(apiLink);
      } catch (_unused) {
        setError(errorMessage);
      }

      return Promise.resolve();
    } catch (e) {
      return Promise.reject(e);
    }
  };

  react.useEffect(function () {
    fetchImage();
  }, []);
  return [randomImage, error];
};

var Events;

(function (Events) {
  Events["MOUSEDOWN"] = "mousedown";
  Events["TOUCHSTART"] = "touchstart";
})(Events || (Events = {}));

var useClickOutside = function useClickOutside(ref, handler) {
  react.useEffect(function () {
    var listener = function listener(event) {
      if (!ref.current || ref.current.contains(event.target)) {
        return;
      }

      handler(event);
    };

    document.addEventListener('mousedown', listener);
    document.addEventListener('touchstart', listener);
    return function () {
      document.removeEventListener('mousedown', listener);
      document.removeEventListener('touchstart', listener);
    };
  }, [ref, handler]);
};

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

var Status;

(function (Status) {
  Status["Rejected"] = "rejected";
  Status["Resolved"] = "resolved";
  Status["Pending"] = "pending";
  Status["Idle"] = "idle";
})(Status || (Status = {}));

var ErrorMessages;

(function (ErrorMessages) {
  ErrorMessages["UnableToReachLocation"] = "unableToReachLocation";
  ErrorMessages["GeoIsNotSupported"] = "geoIsNotSupported";
})(ErrorMessages || (ErrorMessages = {}));

var Type;

(function (Type) {
  Type["Error"] = "error";
  Type["Success"] = "success";
  Type["Started"] = "started";
})(Type || (Type = {}));

var geoReducer = function geoReducer(state, action) {
  switch (action.type) {
    case Type.Error:
      return _extends({}, state, {
        status: Status.Rejected,
        error: action.error
      });

    case Type.Success:
      return _extends({}, state, {
        status: Status.Resolved,
        position: action.position
      });

    case Type.Started:
      return _extends({}, state, {
        status: Status.Pending
      });

    default:
      throw new Error(ErrorMessages.UnableToReachLocation);
  }
};

var initialState = {
  status: Status.Idle,
  position: null,
  error: null
};

var useGeoPosition = function useGeoPosition() {
  var _useReducer = react.useReducer(geoReducer, initialState),
      state = _useReducer[0],
      dispatch = _useReducer[1];

  react.useEffect(function () {
    if (!navigator.geolocation) {
      dispatch({
        type: Type.Error,
        error: ErrorMessages.GeoIsNotSupported
      });
      return;
    }

    dispatch({
      type: Type.Started
    });
    var geoWatch = navigator.geolocation.watchPosition(function (position) {
      return dispatch({
        type: Type.Success,
        position: position
      });
    }, function (error) {
      return dispatch({
        type: Type.Error,
        error: error
      });
    });
    return function () {
      return navigator.geolocation.clearWatch(geoWatch);
    };
  }, []);
  var status = state.status;
  return _extends({
    isLoading: status === Status.Idle || status === Status.Pending,
    isResolved: status === Status.Resolved,
    isRejected: status === Status.Rejected
  }, state);
};

var useIdle = function useIdle(options) {
  var _useState = react.useState(false),
      isIdle = _useState[0],
      setIsIdle = _useState[1];

  react.useEffect(function () {
    var activityDetector = createActivityDetector(options);
    activityDetector.on('idle', function () {
      return setIsIdle(true);
    });
    activityDetector.on('active', function () {
      return setIsIdle(false);
    });
    return function () {
      return activityDetector.stop();
    };
  }, []);
  return isIdle;
};

var useScrollPosition = function useScrollPosition() {
  var _useState = react.useState(null),
      scrollPosition = _useState[0],
      setScrollPosition = _useState[1];

  react.useEffect(function () {
    var handleScroll = function handleScroll() {
      return setScrollPosition(window.scrollY);
    };

    document.addEventListener('scroll', handleScroll);
    return function () {
      return document.removeEventListener('scroll', handleScroll);
    };
  }, []);
  return scrollPosition;
};

var useMousePosition = function useMousePosition() {
  var _useState = react.useState({
    posX: null,
    posY: null
  }),
      mousePosition = _useState[0],
      setMousePosition = _useState[1];

  var handleMouseMove = react.useCallback(function (e) {
    setMousePosition({
      posX: e.pageX,
      posY: e.pageY
    });
  }, []);
  react.useEffect(function () {
    window.addEventListener('mousemove', handleMouseMove);
    return function () {
      window.removeEventListener('mousemove', handleMouseMove);
    };
  }, []);
  return mousePosition;
};

var useEventListener = function useEventListener(element, eventName, handler) {
  var handlerRef = react.useRef(handler);
  handlerRef.current = handler;
  react.useEffect(function () {
    if (!element) return;

    var eventListener = function eventListener(evt) {
      return handlerRef.current.call(element, evt);
    };

    element.addEventListener(eventName, eventListener);
    return function () {
      element.removeEventListener(eventName, eventListener);
    };
  }, [eventName, element]);
};

var Type$1;

(function (Type) {
  Type["Error"] = "error";
  Type["Success"] = "success";
  Type["Started"] = "started";
})(Type$1 || (Type$1 = {}));

var Status$1;

(function (Status) {
  Status["Rejected"] = "rejected";
  Status["Resolved"] = "resolved";
  Status["Pending"] = "pending";
  Status["Idle"] = "idle";
})(Status$1 || (Status$1 = {}));

var fetchReducer = function fetchReducer(state, action) {
  switch (action.type) {
    case Type$1.Started:
      return _extends({}, state, {
        status: Status$1.Pending
      });

    case Type$1.Success:
      return _extends({}, state, {
        status: Status$1.Resolved,
        data: action.data
      });

    case Type$1.Error:
      return _extends({}, state, {
        status: Status$1.Rejected,
        error: action.error
      });

    default:
      throw new Error();
  }
};

var initialState$1 = {
  status: Status$1.Idle,
  data: [],
  error: null
};

var useFetch = function useFetch(path) {
  var _useReducer = react.useReducer(fetchReducer, initialState$1),
      state = _useReducer[0],
      dispatch = _useReducer[1];

  var getData = react.useCallback(function () {
    try {
      var _temp2 = _catch(function () {
        return Promise.resolve(fetch(path)).then(function (response) {
          return Promise.resolve(response.json()).then(function (data) {
            dispatch({
              type: Type$1.Success,
              data: data
            });
          });
        });
      }, function (error) {
        dispatch({
          type: Type$1.Error,
          error: error
        });
      });

      return Promise.resolve(_temp2 && _temp2.then ? _temp2.then(function () {}) : void 0);
    } catch (e) {
      return Promise.reject(e);
    }
  }, [path]);
  react.useEffect(function () {
    dispatch({
      type: Type$1.Started
    });
    getData();
  }, []);
  var status = state.status;
  return _extends({
    isLoading: status === Status$1.Idle || status === Status$1.Pending,
    isResolved: status === Status$1.Resolved,
    isRejected: status === Status$1.Rejected
  }, state);
};

var useHover = function useHover() {
  var _useState = react.useState(false),
      isHovered = _useState[0],
      setIsHovered = _useState[1];

  var handleMouseOver = react.useCallback(function () {
    return setIsHovered(true);
  }, []);
  var handleMouseOut = react.useCallback(function () {
    return setIsHovered(false);
  }, []);
  var ref = react.useRef(null);
  var callback = react.useCallback(function (node) {
    if (node) {
      node.removeEventListener('mouseover', handleMouseOver);
      node.removeEventListener('mouseout', handleMouseOut);
    }

    if (node) {
      node.addEventListener('mouseover', handleMouseOver);
      node.addEventListener('mouseout', handleMouseOut);
    }
  }, [handleMouseOver, handleMouseOut]);
  react.useEffect(function () {
    callback(ref.current);
  }, [callback]);
  return [ref, isHovered];
};

var useInterval = function useInterval(callback, delay) {
  react.useEffect(function () {
    var id = setInterval(callback, delay);
    return function () {
      return clearInterval(id);
    };
  }, [delay]);
};

var useLocalStorage = function useLocalStorage(key, initialValue) {
  if (initialValue === void 0) {
    initialValue = '';
  }

  var _useState = react.useState(null),
      error = _useState[0],
      setError = _useState[1];

  var _useState2 = react.useState(function () {
    try {
      var item = window.localStorage.getItem(key);
      return item ? JSON.parse(item) : initialValue;
    } catch (error) {
      setError(error);
      return initialValue;
    }
  }),
      storedValue = _useState2[0],
      setStoredValue = _useState2[1];

  var isFnInstance = react.useCallback(function (value) {
    if (value instanceof Function) {
      return value(storedValue);
    }

    return value;
  }, [storedValue]);
  var deleteItem = react.useCallback(function () {
    try {
      window.localStorage.removeItem(key);
    } catch (error) {
      setError(error);
    }
  }, [key]);
  var setValue = react.useCallback(function (value) {
    try {
      var valueToStore = isFnInstance(value);
      setStoredValue(valueToStore);
      window.localStorage.setItem(key, JSON.stringify(valueToStore));
    } catch (error) {
      setError(error);
    }
  }, [key, isFnInstance]);
  return [storedValue, setValue, deleteItem, error];
};

var useToggle = function useToggle(initialOn) {
  if (initialOn === void 0) {
    initialOn = false;
  }

  var _useState = react.useState(initialOn),
      on = _useState[0],
      setOn = _useState[1];

  var toggle = function toggle() {
    return setOn(!on);
  };

  return {
    on: on,
    toggle: toggle
  };
};

var useKeyPress = function useKeyPress(targetKey) {
  var _useState = react.useState(false),
      keyPressed = _useState[0],
      setKeyPressed = _useState[1];

  var handleKeyDown = react.useCallback(function (_ref) {
    var key = _ref.key;

    if (key === targetKey) {
      setKeyPressed(true);
    }
  }, [targetKey]);
  var handleKeyUp = react.useCallback(function (_ref2) {
    var key = _ref2.key;

    if (key === targetKey) {
      setKeyPressed(false);
    }
  }, [targetKey]);
  react.useEffect(function () {
    window.addEventListener('keydown', handleKeyDown);
    window.addEventListener('keyup', handleKeyUp);
    return function () {
      window.removeEventListener('keydown', handleKeyDown);
      window.removeEventListener('keyup', handleKeyUp);
    };
  }, [handleKeyDown, handleKeyUp]);
  return keyPressed;
};

exports.useClickOutside = useClickOutside;
exports.useEventListener = useEventListener;
exports.useFetch = useFetch;
exports.useGeoPosition = useGeoPosition;
exports.useHelloRomanJoke = useHelloRomanJoke;
exports.useHover = useHover;
exports.useIdle = useIdle;
exports.useInterval = useInterval;
exports.useKeyPress = useKeyPress;
exports.useLocalStorage = useLocalStorage;
exports.useMousePosition = useMousePosition;
exports.usePageWidth = usePageWidth;
exports.useScrollPosition = useScrollPosition;
exports.useToggle = useToggle;
exports.useUnsplash = useUnsplash;
//# sourceMappingURL=index.js.map
